// Search.cs
// Written by David J. Stein, Esq. See "License.html" for terms and conditions of use.
//
// This module contains the code for the main search window.

#region Using declarations

using System;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

#endregion

namespace SearchNS {

	public partial class Search : Form {

		#region Variables, initialization code, and Main

		private ArrayList arraySearchEngines;
		public static Regex regexSearchEngine;

		[STAThread]
		static void Main() {
			Application.EnableVisualStyles();
			Application.SetCompatibleTextRenderingDefault(false);
			regexSearchEngine = new Regex("<SearchEngine>.*?<Name>(.*?)</Name>.*?<URL>(.*?)</URL>.*?</SearchEngine>", RegexOptions.Multiline);
			Application.Run(new Search());
		}

		public Search() {
			InitializeComponent();
		}

		#endregion

		#region Event handlers

		private void Search_Load(object sender, EventArgs e) {
			LoadSearchEnginesComboBox();
		}

		private void btnSearch_Click(object sender, EventArgs e) {
			foreach (SearchEngine engine in arraySearchEngines) {
				if (engine.strName == cmbSearchEngines.Text) {
					string strURL = engine.strURL.Replace("SEARCHTEXT", txtSearch.Text.Trim().Replace("%", "%25").Replace(" ", "%20").Replace("!", "%21").Replace("\"", "%22").Replace("#", "%23").Replace("$", "%24").Replace("&", "%26").Replace("'", "%27").Replace("(", "%28").Replace(")", "%29").Replace("*", "%2A").Replace("+", "%2B").Replace(",", "%2C").Replace("-", "%2D").Replace(".", "%2E").Replace("/", "%2F").Replace(":", "%3A").Replace(";", "%3B").Replace("<", "%3C").Replace("=", "%3D").Replace(">", "%3E").Replace("?", "%3F").Replace("@", "%40").Replace("[", "%5B").Replace("\\", "%5C").Replace("]", "%5D").Replace("^", "%5E").Replace("`", "%60").Replace("{", "%7B").Replace("|", "%7C").Replace("}", "%7D").Replace("~", "%7E"));
					System.Diagnostics.Process.Start(strURL);
					break;
				}
			}
			Close();
		}

		private void btnConfigure_Click(object sender, EventArgs e) {
			new ConfigureSearch().ShowDialog();
			LoadSearchEnginesComboBox();
		}

		private void btnClose_Click(object sender, EventArgs e) {
			Close();
		}

		#endregion

		#region Helper functions

		internal static ArrayList ReadSearchEnginesXML() {
			// This function is declared static so that ConfigureSearch can also use it.
			ArrayList arraySearchEngines = new ArrayList();
			string strFilename = Application.StartupPath + "\\Search Engines.xml";
			if (System.IO.File.Exists(strFilename) == true) {
				try {
					string strSearchEngines = System.IO.File.ReadAllText(strFilename).Replace("\r", "").Replace("\n", "");
					MatchCollection matchesEngines = regexSearchEngine.Matches(strSearchEngines);
					foreach (Match matchEngine in matchesEngines)
						arraySearchEngines.Add(new SearchEngine(matchEngine.Groups[1].Value, matchEngine.Groups[2].Value));
				} catch (Exception exception) {
					MessageBox.Show("Search error: " + exception);
				}
			}
			return arraySearchEngines;
		}

		private void LoadSearchEnginesComboBox() {
			arraySearchEngines = Search.ReadSearchEnginesXML();
			if (arraySearchEngines.Count == 0)
				arraySearchEngines.Add(new SearchEngine("Google", "http://www.google.com/search?q=SEARCHTEXT"));
			cmbSearchEngines.BeginUpdate();
			cmbSearchEngines.Items.Clear();
			foreach (SearchEngine engine in arraySearchEngines)
				cmbSearchEngines.Items.Add(engine.strName);
			cmbSearchEngines.Text = ((SearchEngine) arraySearchEngines[0]).strName;
			cmbSearchEngines.EndUpdate();
			cmbSearchEngines.Refresh();
		}

		#endregion

	}

	internal class SearchEngine {
		// This class is just a data structure.

		#region Variables and initialization code

		internal string strName;
		internal string strURL;

		internal SearchEngine(string strName, string strURL) {
			this.strName = strName;
			this.strURL = strURL;
		}

		#endregion

	}

}
